@@ -43,7 +43,7 @@ fn variable_base_test_with_bls12() {
 
     let mut rng = XorShiftRng::seed_from_u64(234872845u64);
 
-    let v = (0..SAMPLES).map(|_| Fr::rand(&mut rng).into_repr()).collect::<Vec<_>>();
+    let v = (0..SAMPLES).map(|_| Fr::rand(&mut rng).to_repr()).collect::<Vec<_>>();
     let g = (0..SAMPLES)
         .map(|_| G1Projective::rand(&mut rng).into_affine())
         .collect::<Vec<_>>();
@@ -61,7 +61,7 @@ fn variable_base_test_with_bls12_unequal_numbers() {
     let mut rng = XorShiftRng::seed_from_u64(234872845u64);
 
     let v = (0..SAMPLES - 1)
-        .map(|_| Fr::rand(&mut rng).into_repr())
+        .map(|_| Fr::rand(&mut rng).to_repr())
         .collect::<Vec<_>>();
     let g = (0..SAMPLES)
         .map(|_| G1Projective::rand(&mut rng).into_affine())
