@@ -376,15 +376,15 @@ mod tests {
         let output: Vec<Fq> = run_roundtrip("mul_test", &inputs[..]);
         for (input, output) in inputs.iter().zip(output.iter()) {
             let rust_out = input[0] * &input[1];
-            let output = output.into_repr_raw();
-            let rust_out = rust_out.into_repr_raw();
+            let output = output.to_repr_unchecked();
+            let rust_out = rust_out.to_repr_unchecked();
 
             if rust_out != output {
                 eprintln!("test failed: {:?} != {:?}", rust_out.as_ref(), output.as_ref());
                 eprintln!(
                     "inputs {:?}, {:?}",
-                    input[0].into_repr_raw().as_ref(),
-                    input[1].into_repr_raw().as_ref()
+                    input[0].to_repr_unchecked().as_ref(),
+                    input[1].to_repr_unchecked().as_ref()
                 );
                 assert_eq!(rust_out.as_ref(), output.as_ref());
             }
@@ -398,12 +398,12 @@ mod tests {
         let output: Vec<Fq> = run_roundtrip("sqr_test", &inputs[..]);
         for (input, output) in inputs.iter().zip(output.iter()) {
             let rust_out = input[0].square();
-            let output = output.into_repr_raw();
-            let rust_out = rust_out.into_repr_raw();
+            let output = output.to_repr_unchecked();
+            let rust_out = rust_out.to_repr_unchecked();
 
             if rust_out != output {
                 eprintln!("test failed: {:?} != {:?}", rust_out.as_ref(), output.as_ref());
-                eprintln!("inputs {:?}", input[0].into_repr_raw().as_ref());
+                eprintln!("inputs {:?}", input[0].to_repr_unchecked().as_ref());
                 assert_eq!(rust_out.as_ref(), output.as_ref());
             }
         }
@@ -417,15 +417,15 @@ mod tests {
 
         for (input, output) in inputs.iter().zip(output.iter()) {
             let rust_out = input[0] + &input[1];
-            let output = output.into_repr_raw();
-            let rust_out = rust_out.into_repr_raw();
+            let output = output.to_repr_unchecked();
+            let rust_out = rust_out.to_repr_unchecked();
 
             if rust_out != output {
                 eprintln!("test failed: {:?} != {:?}", rust_out.as_ref(), output.as_ref());
                 eprintln!(
                     "inputs {:?}, {:?}",
-                    input[0].into_repr_raw().as_ref(),
-                    input[1].into_repr_raw().as_ref()
+                    input[0].to_repr_unchecked().as_ref(),
+                    input[1].to_repr_unchecked().as_ref()
                 );
                 assert_eq!(rust_out.as_ref(), output.as_ref());
             }
@@ -519,14 +519,14 @@ mod tests {
 
         for (input, output) in inputs.iter().zip(output.iter()) {
             let rust_out = input[0] - &input[1];
-            let output = output.into_repr_raw();
-            let rust_out = rust_out.into_repr_raw();
+            let output = output.to_repr_unchecked();
+            let rust_out = rust_out.to_repr_unchecked();
             if rust_out != output {
                 eprintln!("test failed: {:?} != {:?}", rust_out.as_ref(), output.as_ref());
                 eprintln!(
                     "inputs {:?}, {:?}",
-                    input[0].into_repr_raw().as_ref(),
-                    input[1].into_repr_raw().as_ref()
+                    input[0].to_repr_unchecked().as_ref(),
+                    input[1].to_repr_unchecked().as_ref()
                 );
                 assert_eq!(rust_out.as_ref(), output.as_ref());
             }
