@@ -64,7 +64,7 @@ pub struct Proof<E: PairingEngine> {
 
 impl<E: PairingEngine> ToBytes for Proof<E> {
     #[inline]
-    fn write<W: Write>(&self, mut writer: W) -> IoResult<()> {
+    fn write_le<W: Write>(&self, mut writer: W) -> IoResult<()> {
         match self.compressed {
             true => self.write_compressed(&mut writer),
             false => self.write_uncompressed(&mut writer),
@@ -74,7 +74,7 @@ impl<E: PairingEngine> ToBytes for Proof<E> {
 
 impl<E: PairingEngine> FromBytes for Proof<E> {
     #[inline]
-    fn read<R: Read>(mut reader: R) -> IoResult<Self> {
+    fn read_le<R: Read>(mut reader: R) -> IoResult<Self> {
         Self::read(&mut reader)
     }
 }
@@ -108,9 +108,9 @@ impl<E: PairingEngine> Proof<E> {
     /// Serialize the proof into bytes in uncompressed form, for storage
     /// on disk or transmission over the network.
     pub fn write_uncompressed<W: Write>(&self, mut writer: W) -> IoResult<()> {
-        self.a.write(&mut writer)?;
-        self.b.write(&mut writer)?;
-        self.c.write(&mut writer)
+        self.a.write_le(&mut writer)?;
+        self.b.write_le(&mut writer)?;
+        self.c.write_le(&mut writer)
     }
 
     /// Deserialize the proof from compressed bytes.
@@ -120,9 +120,9 @@ impl<E: PairingEngine> Proof<E> {
 
     /// Deserialize the proof from uncompressed bytes.
     pub fn read_uncompressed<R: Read>(mut reader: R) -> IoResult<Self> {
-        let a: E::G1Affine = FromBytes::read(&mut reader)?;
-        let b: E::G2Affine = FromBytes::read(&mut reader)?;
-        let c: E::G1Affine = FromBytes::read(&mut reader)?;
+        let a: E::G1Affine = FromBytes::read_le(&mut reader)?;
+        let b: E::G2Affine = FromBytes::read_le(&mut reader)?;
+        let c: E::G1Affine = FromBytes::read_le(&mut reader)?;
 
         Ok(Self {
             a,
@@ -181,14 +181,24 @@ pub struct VerifyingKey<E: PairingEngine> {
 }
 
 impl<E: PairingEngine> ToBytes for VerifyingKey<E> {
-    fn write<W: Write>(&self, mut writer: W) -> IoResult<()> {
-        self.write(&mut writer)
+    /// Writes the verifying key into little-endian bytes,
+    /// for storage on disk or transmission over the network.
+    fn write_le<W: Write>(&self, mut writer: W) -> IoResult<()> {
+        self.alpha_g1.write_le(&mut writer)?;
+        self.beta_g2.write_le(&mut writer)?;
+        self.gamma_g2.write_le(&mut writer)?;
+        self.delta_g2.write_le(&mut writer)?;
+        (self.gamma_abc_g1.len() as u32).write_le(&mut writer)?;
+        for g in &self.gamma_abc_g1 {
+            g.write_le(&mut writer)?;
+        }
+        Ok(())
     }
 }
 
 impl<E: PairingEngine> FromBytes for VerifyingKey<E> {
     #[inline]
-    fn read<R: Read>(mut reader: R) -> IoResult<Self> {
+    fn read_le<R: Read>(mut reader: R) -> IoResult<Self> {
         Self::read(&mut reader)
     }
 }
@@ -218,31 +228,17 @@ impl<E: PairingEngine> Default for VerifyingKey<E> {
 }
 
 impl<E: PairingEngine> VerifyingKey<E> {
-    /// Serialize the verification key into bytes, for storage on disk
-    /// or transmission over the network.
-    pub fn write<W: Write>(&self, mut writer: W) -> IoResult<()> {
-        self.alpha_g1.write(&mut writer)?;
-        self.beta_g2.write(&mut writer)?;
-        self.gamma_g2.write(&mut writer)?;
-        self.delta_g2.write(&mut writer)?;
-        (self.gamma_abc_g1.len() as u32).write(&mut writer)?;
-        for g in &self.gamma_abc_g1 {
-            g.write(&mut writer)?;
-        }
-        Ok(())
-    }
-
     /// Deserialize the verification key from bytes.
     pub fn read<R: Read>(mut reader: R) -> IoResult<Self> {
-        let alpha_g1: E::G1Affine = FromBytes::read(&mut reader)?;
-        let beta_g2: E::G2Affine = FromBytes::read(&mut reader)?;
-        let gamma_g2: E::G2Affine = FromBytes::read(&mut reader)?;
-        let delta_g2: E::G2Affine = FromBytes::read(&mut reader)?;
+        let alpha_g1: E::G1Affine = FromBytes::read_le(&mut reader)?;
+        let beta_g2: E::G2Affine = FromBytes::read_le(&mut reader)?;
+        let gamma_g2: E::G2Affine = FromBytes::read_le(&mut reader)?;
+        let delta_g2: E::G2Affine = FromBytes::read_le(&mut reader)?;
 
-        let gamma_abc_g1_len: u32 = FromBytes::read(&mut reader)?;
+        let gamma_abc_g1_len: u32 = FromBytes::read_le(&mut reader)?;
         let mut gamma_abc_g1: Vec<E::G1Affine> = Vec::with_capacity(gamma_abc_g1_len as usize);
         for _ in 0..gamma_abc_g1_len {
-            let gamma_abc_g1_element: E::G1Affine = FromBytes::read(&mut reader)?;
+            let gamma_abc_g1_element: E::G1Affine = FromBytes::read_le(&mut reader)?;
             gamma_abc_g1.push(gamma_abc_g1_element);
         }
 
@@ -270,59 +266,55 @@ pub struct ProvingKey<E: PairingEngine> {
 }
 
 impl<E: PairingEngine> ToBytes for ProvingKey<E> {
-    fn write<W: Write>(&self, mut writer: W) -> IoResult<()> {
-        self.write(&mut writer)
+    /// Serialize the proving key into little-endian bytes.
+    fn write_le<W: Write>(&self, mut writer: W) -> IoResult<()> {
+        self.vk.write_le(&mut writer)?;
+
+        self.beta_g1.write_le(&mut writer)?;
+
+        self.delta_g1.write_le(&mut writer)?;
+
+        (self.a_query.len() as u32).write_le(&mut writer)?;
+        for g in &self.a_query[..] {
+            g.write_le(&mut writer)?;
+        }
+
+        (self.b_g1_query.len() as u32).write_le(&mut writer)?;
+        for g in &self.b_g1_query[..] {
+            g.write_le(&mut writer)?;
+        }
+
+        (self.b_g2_query.len() as u32).write_le(&mut writer)?;
+        for g in &self.b_g2_query[..] {
+            g.write_le(&mut writer)?;
+        }
+
+        (self.h_query.len() as u32).write_le(&mut writer)?;
+        for g in &self.h_query[..] {
+            g.write_le(&mut writer)?;
+        }
+
+        (self.l_query.len() as u32).write_le(&mut writer)?;
+        for g in &self.l_query[..] {
+            g.write_le(&mut writer)?;
+        }
+
+        Ok(())
     }
 }
 
 impl<E: PairingEngine> FromBytes for ProvingKey<E> {
     #[inline]
-    fn read<R: Read>(mut reader: R) -> IoResult<Self> {
+    fn read_le<R: Read>(mut reader: R) -> IoResult<Self> {
         Self::read(&mut reader, false)
     }
 }
 
 impl<E: PairingEngine> ProvingKey<E> {
-    /// Serialize the parameters to bytes.
-    pub fn write<W: Write>(&self, mut writer: W) -> IoResult<()> {
-        self.vk.write(&mut writer)?;
-
-        self.beta_g1.write(&mut writer)?;
-
-        self.delta_g1.write(&mut writer)?;
-
-        (self.a_query.len() as u32).write(&mut writer)?;
-        for g in &self.a_query[..] {
-            g.write(&mut writer)?;
-        }
-
-        (self.b_g1_query.len() as u32).write(&mut writer)?;
-        for g in &self.b_g1_query[..] {
-            g.write(&mut writer)?;
-        }
-
-        (self.b_g2_query.len() as u32).write(&mut writer)?;
-        for g in &self.b_g2_query[..] {
-            g.write(&mut writer)?;
-        }
-
-        (self.h_query.len() as u32).write(&mut writer)?;
-        for g in &self.h_query[..] {
-            g.write(&mut writer)?;
-        }
-
-        (self.l_query.len() as u32).write(&mut writer)?;
-        for g in &self.l_query[..] {
-            g.write(&mut writer)?;
-        }
-
-        Ok(())
-    }
-
     /// Deserialize the public parameters from bytes.
     pub fn read<R: Read>(mut reader: R, checked: bool) -> IoResult<Self> {
         let read_g1_affine = |mut reader: &mut R| -> IoResult<E::G1Affine> {
-            let g1_affine: E::G1Affine = FromBytes::read(&mut reader)?;
+            let g1_affine: E::G1Affine = FromBytes::read_le(&mut reader)?;
 
             if checked && !g1_affine.is_in_correct_subgroup_assuming_on_curve() {
                 return Err(io::Error::new(
@@ -335,7 +327,7 @@ impl<E: PairingEngine> ProvingKey<E> {
         };
 
         let read_g2_affine = |mut reader: &mut R| -> IoResult<E::G2Affine> {
-            let g2_affine: E::G2Affine = FromBytes::read(&mut reader)?;
+            let g2_affine: E::G2Affine = FromBytes::read_le(&mut reader)?;
 
             if checked && !g2_affine.is_in_correct_subgroup_assuming_on_curve() {
                 return Err(io::Error::new(
@@ -349,35 +341,35 @@ impl<E: PairingEngine> ProvingKey<E> {
 
         let vk = VerifyingKey::<E>::read(&mut reader)?;
 
-        let beta_g1: E::G1Affine = FromBytes::read(&mut reader)?;
+        let beta_g1: E::G1Affine = FromBytes::read_le(&mut reader)?;
 
-        let delta_g1: E::G1Affine = FromBytes::read(&mut reader)?;
+        let delta_g1: E::G1Affine = FromBytes::read_le(&mut reader)?;
 
-        let a_query_len: u32 = FromBytes::read(&mut reader)?;
+        let a_query_len: u32 = FromBytes::read_le(&mut reader)?;
         let mut a_query = Vec::with_capacity(a_query_len as usize);
         for _ in 0..a_query_len {
             a_query.push(read_g1_affine(&mut reader)?);
         }
 
-        let b_g1_query_len: u32 = FromBytes::read(&mut reader)?;
+        let b_g1_query_len: u32 = FromBytes::read_le(&mut reader)?;
         let mut b_g1_query = Vec::with_capacity(b_g1_query_len as usize);
         for _ in 0..b_g1_query_len {
             b_g1_query.push(read_g1_affine(&mut reader)?);
         }
 
-        let b_g2_query_len: u32 = FromBytes::read(&mut reader)?;
+        let b_g2_query_len: u32 = FromBytes::read_le(&mut reader)?;
         let mut b_g2_query = Vec::with_capacity(b_g2_query_len as usize);
         for _ in 0..b_g2_query_len {
             b_g2_query.push(read_g2_affine(&mut reader)?);
         }
 
-        let h_query_len: u32 = FromBytes::read(&mut reader)?;
+        let h_query_len: u32 = FromBytes::read_le(&mut reader)?;
         let mut h_query = Vec::with_capacity(h_query_len as usize);
         for _ in 0..h_query_len {
             h_query.push(read_g1_affine(&mut reader)?);
         }
 
-        let l_query_len: u32 = FromBytes::read(&mut reader)?;
+        let l_query_len: u32 = FromBytes::read_le(&mut reader)?;
         let mut l_query = Vec::with_capacity(l_query_len as usize);
         for _ in 0..l_query_len {
             l_query.push(read_g1_affine(&mut reader)?);
